const { async } = require('regenerator-runtime');

module.exports = async function (RED) {
    let ddb = await import('dolphindb')
    let { DDB } = await import('dolphindb')
    let { DdbString } = await import('dolphindb')
    /**
     * Config node. 
     */
    function DolphinConfigNode(n) {
        RED.nodes.createNode(this, n);
        this.url = n.url
        this.name = n.name
        this.client = new DDB(this.url, {
            // 是否在建立连接后自动登录，默认 `true`
            autologin: true,

            // DolphinDB 登录用户名，默认 `'admin'`
            username: this.credentials.username,

            // DolphinDB 登录密码，默认 `'123456'`
            password: this.credentials.password,

            // 设置 python session flag，默认 `false`
            python: false,

            // 设置该选项后，该数据库连接只用于流数据，详细用法见后文 `5. 流数据`
            streaming: undefined
        })
    }
    RED.nodes.registerType("dolphindb", DolphinConfigNode, {
        credentials: {
            username: { type: "text" },
            password: { type: "password" }
        }
    });


    /**
     * Output node to write to a single dolphindb measurement
     */
    function DolphinOutNode(n) {
        RED.nodes.createNode(this, n);

        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        this.database = n.database;

        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }

        var node = this;

        var client = this.dolphindbConfig.client;

        node.on("input", function (msg, send, done) {
            let data = msg.payload
            const measurement = this.database;
            const tags = Object.entries(data.tags).map(([key, value]) => `${key}=${value}`).join(",");
            const fields = Object.entries(data.fields).map(([key, value]) => `${key}=${value}`).join(",");
            const time = new Date(data.time).getTime() * 1000000; // Convert to nanoseconds

            const param = `${measurement},${tags} ${fields} ${time}`;
            client.call('userDefinedAppend', [new DdbString("INFLUXDB"), new DdbString(param)]).then(() => {
                done();
            }).catch(function (err) {
                msg.dolphin_error = err
                // this.error(err);
                done(err);
            });

        });
    }

    RED.nodes.registerType("无模式写入", DolphinOutNode);

    // 函数结点
    function DolphinFuncNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;
        this.function = n.function
        this.params = n.params
        this.script = n.script

        console.log(n)

        // 处理脚本
        const paramStr = Object.entries(this.params)
            .map(([param, value]) => {
                return `${param}=${JSON.stringify(value)}`;
                // return `${param}=${value}`;

            })
            .join("; ");
        this.script = paramStr + ";" + this.script
        console.log(this.script)
        var node = this;

        node.on('input', async function (msg, send, done) {
            // console.log(msg)
            client.eval(node.script).then(res => {
                // console.log(res)
                msg.payload = JSON.parse(res.value)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                // console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("dolphindb func", DolphinFuncNode);



    // 流表结点
    function DolphinStreamNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;


        this.params = n.params
        this.name = n.name
        this.capacity = n.capacity
        this.size = n.size
        this.sendResult = n.sendResult
        const paramNames = this.params.map(param => "`" + param.paramName).join("");
        const paramTypeString = this.params.map(param => param.paramType).join(",");
        // const script = `share streamTable(${this.capacity}:${this.size}, ${paramNames}, [${paramTypeString}]) as ${this.name}`;
        // var node = this;
        // // console.log(script)
        // node.on('input', async function (msg, send, done) {
        //     client.eval(script).then(res => {
        //         // console.log("hello world")
        //         // console.log(res)
        //         // 返回执行结果
        //         send(msg)
        //         done()
        //     }).catch(err => {
        //         console.log(err)
        //         done(err)
        //     })
        //
        // });


        const script = `
        topics = getStreamingStat().pubTables;
        unsubscribeTableName = "${this.name}"
        for(topic in topics){
            tableName = topic.tableName
            actionName = topic.actions
            if(actionName[0]=="["){
                actionName = substr(actionName,1,strlen(actionName)-2)
            }
            actions = split(actionName,",")
            for(action in actions){
                if (tableName == unsubscribeTableName){
                    try{unsubscribeTable(tableName=tableName,actionName=action);}catch(ex){print(ex)};
                }
            }

        };
        try{dropStreamTable(\`${this.name})}catch(ex){print(ex)}
        share streamTable(${this.capacity}:${this.size}, ${paramNames}, [${paramTypeString}]) as ${this.name};\n`;
        var node = this;
        // console.log(this.name, "流表创建的脚本信息:")
        // console.log(script)
        client.eval(script).then(res => {
            console.log(this.name, "流表创建成功:")
            console.log(res)
            // 返回执行结果
            if (node.sendResult) {
                try {
                    // 新建流数据连接配置
                    let sddb = new DDB(client.url, {
                        autologin: true,
                        username: 'admin',
                        password: '123456',
                        streaming: {
                            table: node.name,

                            // 流数据处理回调, message 的类型是 StreamingData
                            handler(message) {
                                // console.log("流表的信息：")
                                // console.log(this)
                                console.log("回调消息返回:")
                                console.log(message.data)

                                let msg = {}
                                returnMsg = JSON.parse(JSON.stringify(message.data));
                                msg.payload = returnMsg
                                node.send(msg)
                            }
                        }
                    })
                    // 建立连接
                    sddb.connect()
                } catch (error) {
                    console.log("订阅回调失败")
                    console.log(error)
                    return
                }

            }
        }).catch(err => {
            console.log(this.name, "流表创建有错误:")
            console.log(err)
        })
        // console.log(client.url)

        node.on('input', async function (msg, send, done) {
            // console.log(msg)
            let paramValueString = ""
            try {
                paramValueString = node.params.map(param => {
                    if (param.paramType == "SYMBOL") {
                        paramName = param.paramName
                        if (msg.payload.params[paramName] != undefined) {
                            // console.log(typeof msg.payload.params[param.paramName])

                            return '`' + msg.payload.params[paramName];
                        }
                        else {
                            return;
                        }
                    }
                    else {
                        paramName = param.paramName
                        if (msg.payload.params[paramName] != undefined) {
                            // console.log(typeof msg.payload.params[param.paramName])

                            return msg.payload.params[paramName];
                        }
                        else {
                            return;
                        }
                    }
                }).join(',');
            } catch (error) {
                send("格式不正确")
                done()
                return
            }

            var addScript = `tableInsert(${this.name}, (${paramValueString}));`
            // console.log(addScript)
            client.eval(addScript).then(res => {
                // console.log(res)
                send(res)
                done()
                // 返回执行结果
            }).catch(err => {
                // console.log(err)
                done(err)
            })
        });


    }
    RED.nodes.registerType("流表创建", DolphinStreamNode);


    //流数据引擎案例节点
    function DolphinEngineCaseNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.function = n.function
        this.params = n.params
        this.script = n.script
        // console.log("为什么script这么长",this.script)
        this.excuteOnce = n.excuteOnce
        let excuteCount = 0
        // console.log(n)

        // 处理脚本
        const paramStr = Object.entries(this.params)
            .map(([param, value]) => {
                return `${param}=${JSON.stringify(value)}`;
                // return `${param}=${value}`;

            })
            .join("; ");
        this.script = paramStr + ";" + this.script
        // console.log(this.script)
        var node = this;

        node.on('input', function (msg, send, done) {
            if (node.excuteOnce && excuteCount > 0) {
                return
            }
            // console.log(node.script)
            client.eval(node.script).then(res => {
                excuteCount += 1
                console.log("函数执行成功信息:")
                console.log(res)
                msg.payload = res.value
                // 返回执行结果
                // send(msg)
                done()
            }).catch(err => {
                console.log("函数执行失败信息:")
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("dolphindb engine case", DolphinEngineCaseNode);
    function parseMessage(inputString) {
        // 使用正则表达式匹配各个部分
        const regex = /(\w+)\[(\d+)\]\(\[?([^[\]]*)\]?\)/g;
        let match;
        let timestampValues = [];
        let lastSymbolValues = [];

        while ((match = regex.exec(inputString))) {
            const key = match[1];
            const count = parseInt(match[2]);
            const valueString = match[3];

            let values = [];

            // 解析每个部分中的值
            if (valueString.includes(",")) {
                values = valueString.split(',').map(v => v.trim().replace(/(^['"])|(['"]$)/g, ''));
            } else if (valueString) {
                values = [valueString.trim().replace(/(^['"])|(['"]$)/g, '')];
            }

            if (key === 'timestamp') {
                timestampValues = values;
            } else if (key === 'symbol') {
                lastSymbolValues = values;
            }
        }
        let msg = {}
        msg.payload = {}
        msg.payload.result = {}
        msg.payload.result.timestamp = timestampValues
        msg.payload.result.info = lastSymbolValues
        return msg
    }


    // 脚本执行结点
    function DolphinScriptNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        // console.log(n.info)
        this.info = n.info
        var node = this;

        node.on('input', async function (msg, send, done) {
            client.eval(node.info).then(res => {
                // console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("dolphindb executefunc", DolphinScriptNode);

    // 流引擎
    function DolphinEngineNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name = n.name
        this.selectedIndex = n.selectedIndex
        this.windowSize = n.windowSize
        this.step = n.step
        this.metrics = n.metrics
        this.dummyTable = n.dummyTable
        this.outputTable = n.outputTable
        this.timeColumn = n.timeColumn
        this.keyColumn = n.keyColumn
        this.tableName = n.tableName
        this.actionName = n.actionName
        this.engineType = n.engineType
        var node = this
        const keyColumns = this.keyColumn.split(" ")
        this.keyColumn = keyColumns.map(column => "`" + column).join(",");
        var script = `hour = create${this.engineType}Engine(name="${this.name}",  windowSize=${this.windowSize}, step=${this.step}, metrics=<[${this.metrics}]>, dummyTable=${this.dummyTable}, outputTable=${this.outputTable}, timeColumn=\`${this.timeColumn}, keyColumn=[${this.keyColumn}])
        subscribeTable(tableName="${this.tableName}", actionName="${this.actionName}", offset=0, handler=append!{${this.name}}, msgAsTable=true, batchSize=50, throttle=1, hash=0)`
        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("流数据引擎", DolphinEngineNode);

    //文件加载节点
    function DolphinLoadTextNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name;
        this.textName=n.textName;
        this.tableName=n.tableName;

        var node=this
        var script=`${this.tableName}=loadText("${this.textName}"); `
        console.log(script)

        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });

    }
    RED.nodes.registerType("数据加载",DolphinLoadTextNode);

    function DolphinFormatConversionNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name
        this.tableName=n.tableName
        this.colName=n.colName
        this.formatStr=n.formatStr
        var node = this

        var func=`def changeFormat(tableName,colName,formatStr){ 
                                pt=objByName(tableName); 
                                schema=pt.schema().colDefs;
                                type=select typeString from schema where name=colName;
                                if(type.cell(0,0)=="DOUBLE"){
                                    print("yes");
                                    pt[colName]=double(format(pt[colName],formatStr));
                                }else if(type.cell(0,0)=="DATE"){
                                    pt[colName]=date(format(pt[colName],formatStr));
                                }else if(type.cell(0,0)=="DATETIME"){
                                    pt[colName]=datetime(format(pt[colName],formatStr));
                                }else if(type.cell(0,0)=="TIMESTAMP"){
                                    pt[colName]=timestamp(format(pt[colName],formatStr));
                                }else{
                                    print("This type conversion is not supported");
                                }
                            }`;
        var Funcall=`changeFormat("${this.tableName}","${this.colName}","${this.formatStr}");`
        var script =func+Funcall;
        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("格式转换", DolphinFormatConversionNode);


    function DolphinNullFillNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name;
        this.tableName=n.tableName;
        this.nullCol=n.nullCol;
        this.fillNum=n.fillNum;

        var node = this;

        var func=`def typeChange(type,str){
    if(type=="STRING"){ return str;}
    else if(type=="DOUBLE") {return double(str);}
    else if(type=="BOOL") {return bool(str);}
    else if(type=="INT") {return int(str);}
    else if(type=="SHORT") {return short(str);}
    else if(type=="LONG") {return long(str);}
    else if(type=="CHAR") {return char(str);}
    else if(type=="DATE") {return date(str);}
    else if(type=="DATETIME") {return datetime(str);}
    else if(type=="TIMESTAMP") {return timestamp(str);}
    else if(type=="TIME") {return time(str);}
    else if(type=="SYMBOL") {return symbol(str);}
    else if(type=="DURATION") {return duration(str);}
};

def nullFillFunc(tableName,nullCol,fillNum){
    pt=objByName(tableName);
    if(strpos(fillNum,"max")!=-1){
        pt[nullCol]=nullFill(pt[nullCol],max(pt[nullCol])); 
    }else if(strpos(fillNum,"min")!=-1){
        pt[nullCol]=nullFill(pt[nullCol],min(pt[nullCol]));
    }else if(strpos(fillNum,"avg")!=-1){
        pt[nullCol]=nullFill(pt[nullCol],avg(pt[nullCol]));
    }else{
        schema=pt.schema().colDefs;
        type=select typeString from schema where name=nullCol;
        fillNumber=typeChange(type.cell(0,0),fillNum);
        print(fillNumber);
        pt[nullCol]=nullFill(pt[nullCol],fillNumber);
    }
};`;
        var funcall =`nullFillFunc("${this.tableName}",\`${this.nullCol},"${this.fillNum}");`;
        var script =func+funcall;

        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("NULL填充", DolphinNullFillNode);

    function DolphinRemoveDuplicatesNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name;
        this.tableName=n.tableName;
        this.duplicatedCol=n.duplicatedCol;

        var node = this;

        var func=`def removeDuplicates(tableName,colName){
    pt=objByName(tableName);
    shema=pt.schema().colDefs;
    tmp=keyedTable(colName,1:0,shema.name,shema.typeString);
    tmp.append!(pt);
    return tmp;
}`;
        var funcall =`${this.tableName}=removeDuplicates("${this.tableName}",\`${this.duplicatedCol});`;
        var script =func+funcall;

        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("数据去重", DolphinRemoveDuplicatesNode);

    function DolphinDataNormalizationNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name;
        this.tableName=n.tableName;
        this.colName=n.colName;
        this.normalizeType=n.normalizeType;

        var node = this;

        var func=`def minMaxNormalization(tableName,colName){
    pt=objByName(tableName);
    minNum=min(pt[colName]);
    maxNum=max(pt[colName]);
    pt[colName]=(pt[colName]-minNum)/(maxNum-minNum);
}
def ZSoreNormalization(tableName,colName){
    pt=objByName(tableName);
    meanNum=mean(pt[colName]);
    stdNum=std(pt[colName]);
    pt[colName]=(pt[colName]-meanNum)/stdNum;
}
def dataNormalization(tableName,colName,type){
    if(type==1){
        minMaxNormalization(tableName,colName);
    }else if(type==2){
        ZSoreNormalization(tableName,colName);
    }
}`;
        var funcall =`dataNormalization("${this.tableName}",\`${this.colName},${this.normalizeType})`;
        var script =func+funcall;

        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("数据归一化", DolphinDataNormalizationNode);


    function DolphinDataStorageNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;

        this.name=n.name
        this.dbName=n.dbName
        this.partitionType=n.partitionType
        this.partitionPara=n.partitionPara
        this.engine=n.engine
        this.tableName=n.tableName
        this.colName=n.colName
        this.sortColumns=n.sortColumns

        var node = this

        var func=`def inDatabase(dbName,type,partitionScheme,engine,tableName,colName,sortColumns){
                            ddb=database(dbName,type,partitionScheme,,engine);
                            pt=objByName(tableName); 
                            ppt=createPartitionedTable(ddb,pt,tableName,colName,,sortColumns);
                            ppt.append!(pt);
                        }`
        var funcall=`inDatabase("dfs://${this.dbName}",${this.partitionType},[${this.partitionPara}],\`${this.engine},"${this.tableName}",\`${this.colName},\`${this.sortColumns})`
        var script =func+funcall;

            // var script = `db${this.dbName}=database("dfs://${this.dbName}",${this.partitionType},[${this.partitionPara}],,\`${this.engine});
        // createPartitionedTable(db${this.dbName},${this.tableName},"${this.tableName}",\`${this.colName},,\`${this.sortColumns});
        // loadTable("dfs://${this.dbName}","${this.tableName}").append!(${this.tableName});
        // `
        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })
        });
    }
    RED.nodes.registerType("数据入库", DolphinDataStorageNode);


    function DolphinDownsamplingNode(n) {
        RED.nodes.createNode(this, n);
        // console.log(n)

        // 获取配置结点
        this.dolphindb = n.dolphindb;
        this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
        if (!this.dolphindbConfig) {
            this.error(RED._("dolphindb.errors.missingconfig"));
            return;
        }
        var client = this.dolphindbConfig.client;
        this.name=n.name;
        this.dbName=n.dbName;
        this.tableName=n.tableName;
        this.downsamplingType=n.downsamplingType;
        this.downsamplingCol=n.downsamplingCol;
        this.timeColumn=n.timeColumn;
        this.step=n.step;
        var node = this;

        var func=`def downsampling(dbName,tableName,downsamplingType,downsamplingCol,timeColumn,step){ 
    sqlQuery = 'select ' + downsamplingType + '(' + downsamplingCol + ')' + ' from loadTable("' + dbName + '","' + tableName + '")' + ' group by bar(' + timeColumn + ',' + step +') as time'
    return parseExpr(sqlQuery).eval()
}`;
        var funcall=`tmp1=downsampling("dfs://${this.dbName}","${this.tableName}",\`${this.downsamplingType},\`${this.downsamplingCol},\`${this.timeColumn},\`${this.step});
        toJson(tmp1); `;
        var script=func+funcall;

        console.log(script)
        node.on('input', async function (msg, send, done) {
            client.eval(script).then(res => {
                console.log(res)
                // 返回执行结果
                send(msg)
                done()
            }).catch(err => {
                console.log(err)
                done(err)
            })

        });
    }
    RED.nodes.registerType("降采样", DolphinDownsamplingNode);


    // function DolphintestNode(n) {
    //     RED.nodes.createNode(this, n);
    //     // console.log(n)
    //
    //     // 获取配置结点
    //     this.dolphindb = n.dolphindb;
    //     this.dolphindbConfig = RED.nodes.getNode(this.dolphindb);
    //     if (!this.dolphindbConfig) {
    //         this.error(RED._("dolphindb.errors.missingconfig"));
    //         return;
    //     }
    //     var client = this.dolphindbConfig.client;
    //
    //     // console.log(n.info)
    //     this.info = n.info
    //     var node = this;
    //
    //     node.on('input', async function (msg, send, done) {
    //         client.eval(node.info).then(res => {
    //             // console.log(res)
    //             // 返回执行结果
    //             send(msg)
    //             done()
    //         }).catch(err => {
    //             console.log(err)
    //             done(err)
    //         })
    //
    //     });
    // }
    // RED.nodes.registerType("测试节点", DolphintestNode);
}