/// <reference types="node" resolution-mode="require"/>
declare global {
    var my_prototype_defined: boolean;
    interface String {
        readonly width: number;
        /** 截取字符串不超过 width 显示宽度的部分，并保留颜色
            找到并记录能容纳 字符串 + … 的最后一个字符的位置 i_fitted
                若完整的字符串长度超过 width，返回 slice(0, i_fitted + 1) + …
                否则                          返回 this
         */
        truncate(this: string, width: number): string;
        /** pad string to `<width>`
            - character?: `' '`
            - position?: `'right'`
         */
        pad(this: string, width: number, { character, position }?: {
            character?: string;
            position?: 'left' | 'right';
        }): string;
        limit(this: string, width: number, { character, position }?: {
            character?: string;
            position?: 'left' | 'right';
        }): string;
        to_regexp(this: string, preservations?: string, flags?: string): RegExp;
        to_bool(this: string): boolean;
        /** 字符串模式替换
            - pattern: 匹配部分的格式
            - pattern_: 替换后的格式
            - preservations?: `''` 保留的正则表达式字符
            - flags?: `''` 正则匹配选项
            - transformer?: `(name, matched) => matched || ''` placeholder transformer
            - pattern_placeholder?: `/\{.*?\}/g`
            
            ```ts
            'g:/acgn/海贼王/[Skytree][海贼王][One_Piece][893][GB_BIG5_JP][X264_AAC][1080P][CRRIP][天空树双语字幕组].mkv'.refmt(
                '{dirp}/[Skytree][海贼王][{ en_name: \\w+ }][{ episode: \\d+ }][GB_BIG5_JP][{encoding}_AAC][1080P][CRRIP][天空树双语字幕组].{format}',
                'g:/acgn/海贼王/{episode} {encoding}.{format}',
                '\\+',
                'i',
                (name, value) => name === 'episode'  ?  String(+value + 1)  :  value.toLowerCase()
            )
            ```
         */
        refmt(this: string, pattern: string, pattern_: string, preservations?: string, flags?: string, transformer?: (name: string, value: string, placeholders: {
            [name: string]: string;
        }) => string, pattern_placeholder?: RegExp): string;
        /** 字符串模式搜索
            ```ts
            'git+https://github.com/tamino-martinius/node-ts-dedent-123.git'.find(
                '^{protocol:[\\w+]+}://{hostname:[\\w\\.]+}/{username}/{project}-{index:\\d+}.{suffix}', '^', 'i'
            )
            {
                protocol: 'git+https',
                hostname: 'github.com',
                ...
            }
            ```
            
            - preservations?: `''` 保留的正则表达式字符
            - flags?: `''` 正则匹配选项
            - pattern_placeholder?: `/\{.*?\}/g`
        */
        find(this: string, pattern: string, preservations?: string, flags?: string, pattern_placeholder?: RegExp): Record<string, string>;
        /** - type?: `'single'` */
        quote(this: string, type?: keyof typeof quotes | 'psh'): string;
        /** - shape?: `'parenthesis'` */
        bracket(this: string, shape?: keyof typeof brackets): string;
        surround(this: string, left: string, right?: string): string;
        surround_tag(this: string, tag_name: string): string;
        to_lf(this: string): string;
        to_crlf(this: string): string;
        /** 'xxx'.replace(/pattern/g, '')
            如果 pattern 是 string 则在创建 RegExp 时自动加上 flags (默认 'g'), 否则忽略 flags
        */
        rm(this: string, pattern: string | RegExp, flags?: string): string;
        readonly red: string;
        readonly red_: string;
        readonly green: string;
        readonly green_: string;
        readonly yellow: string;
        readonly yellow_: string;
        readonly blue: string;
        readonly blue_: string;
        readonly magenta: string;
        readonly magenta_: string;
        readonly cyan: string;
        readonly cyan_: string;
        readonly grey: string;
        readonly underline: string;
        strip_ansi(this: string): string;
        /** 将 string 划分为行，并去掉最后一个 \n 之后的 '' */
        split_lines(this: string): string[];
        trim_doc_comment(this: string): string;
        split_indent(this: string): {
            indent: number;
            text: string;
        };
        to_base64(this: string): string;
        /** - buffer: `false` 直接返回 Buffer */
        decode_base64(this: string): string;
        decode_base64(this: string, buffer: true): Buffer;
        decode_base64(this: string, buffer?: boolean): string | Buffer;
        space(this: string): string;
        fdir: string;
        /** path.basename, 如:
            - d:/0/aaa.txt -> aaa.txt
            - d:/aaa/ -> aaa/ */
        fname: string;
        /** .txt */
        fext: string;
        to_slash(this: string): string;
        to_backslash(this: string): string;
    }
    interface Date {
        /** - ms?: `false` 显示到 ms */
        to_str(this: Date, ms?: boolean): string;
        to_date_str(this: Date): string;
        /** - ms?: `false` 显示到 ms */
        to_time_str(this: Date, ms?: boolean): string;
    }
    interface Number {
        /** 12.4 KB (1 KB = 1024 B) */
        to_fsize_str(this: number, units?: 'iec' | 'metric'): string;
        to_bin_str(this: number): string;
        to_hex_str(this: number, length?: number): string;
        to_oct_str(this: number): string;
    }
    interface Array<T> {
        last: T;
        log(this: string[], limit?: number): void;
        indent(this: string[], width: number, c?: string): string[];
        indent2to4(this: string[]): string[];
        /**
            - trim_line?: `true`
            - rm_empty_lines?: `true`
            - rm_last_empty_lines?: `false`
        */
        trim_lines(this: string[], { trim_line, rm_empty_lines, rm_last_empty_lines }?: {
            trim_line?: boolean;
            rm_empty_lines?: boolean;
            rm_last_empty_lines?: boolean;
        }): string[];
        trim_license(this: string[]): string[];
        split_indents(this: string[]): {
            indent: number;
            text: string;
        }[];
        /** - append?: `true` 是否在 join 之后增加 \n */
        join_lines(append?: boolean): string;
    }
    interface BigInt {
        toJSON(this: bigint): string;
    }
    interface Error {
        toJSON(this: Error): string;
    }
}
import chalk from 'chalk';
export declare const emoji_regex: RegExp;
export { chalk };
export declare function to_method_property_descriptors(methods: {
    [name: string]: Function;
}): PropertyDescriptorMap;
export declare function to_getter_property_descriptors(getters: {
    [name: string]: Function;
}): PropertyDescriptorMap;
export declare const cjk = "([\u2E80-\u9FFF\uF900-\uFAFF])";
export declare const quotes: {
    single: string;
    double: string;
    backtick: string;
};
export declare const brackets: {
    readonly round: readonly ["(", ")"];
    readonly square: readonly ["[", "]"];
    readonly curly: readonly ["{", "}"];
    readonly pointy: readonly ["<", ">"];
    readonly corner: readonly ["「", "」"];
    readonly fat: readonly ["【", "】"];
    readonly tortoise_shell: readonly ["〔", "〕"];
};
export declare function to_json(obj: any, replacer?: any): string;
export declare function to_json_safely(obj: any, replacer?: any): string;
export declare function is_codepoint_fullwidth(codepoint: number): boolean;
