/// <reference types="node" resolution-mode="require"/>
/// <reference types="node" resolution-mode="require"/>
import { type Server as HttpServer } from 'http';
import { WebSocketServer } from 'ws';
import { default as Koa, type Context, type Next } from 'koa';
import { type UserAgentContext } from 'koa-useragent';
declare module 'koa' {
    interface Request {
        /** 经过 decodeURIComponent 后，在路径重写之前的路径 */
        _path: string;
        body: any;
    }
    interface Context {
        compress: boolean;
        userAgent: UserAgentContext['userAgent'] & {
            isWechat: boolean;
        };
    }
}
import { type Remote, type Headers } from './net.js';
declare module 'http' {
    interface IncomingMessage {
        body?: Buffer;
    }
    interface ServerResponse {
        body?: Buffer;
    }
}
export declare class Server {
    /** proxy 时需要丢弃的 resposne headers */
    static drop_response_headers: Set<string>;
    app: Koa;
    handler: ReturnType<Koa['callback']>;
    port: number;
    server_http: HttpServer;
    server_ws?: WebSocketServer;
    remote?: Remote;
    colors: boolean;
    constructor({ port, remote }?: {
        port?: number;
        remote?: Remote;
    });
    /** start http server and listen */
    start(): Promise<void>;
    stop(): void;
    entry(ctx: Context, next: Next): Promise<void>;
    /**
        parse req.body to request.body
        process request.ip
    */
    parse(ctx: Context): Promise<void>;
    _router(ctx: Context, next: Next): Promise<void>;
    router(ctx: Context): Promise<boolean>;
    logger(ctx: Context): void;
    proxy(ctx: Context, url: string | URL, headers_?: Record<string, string>): Promise<void>;
    static filter_response_headers(headers: Headers): {};
    try_send(ctx: Context, fp: string, { root, log_404, }: {
        root: string;
        log_404?: boolean;
    }): Promise<boolean>;
    /** send file at `path` with the  given `options` to the koa `ctx`. */
    fsend(ctx: Context, path: string, { root, absolute }?: {
        root?: string;
        /** `false` */
        absolute?: boolean;
    }): Promise<string>;
}
